@@ -14,6 +14,7 @@
 
 #include <libqalculate/includes.h>
 #include <libqalculate/util.h>
+#include <sys/time.h>
 #include <pthread.h>
 
 /** @file */
@@ -52,7 +53,7 @@
 *
 * If the program uses autoconf, put the following in configure.in:
 * \code PKG_CHECK_MODULES(QALCULATE, [
-*	libqalculate >= 0.9.8
+*	libqalculate >= 0.9.11
 *	])
 * AC_SUBST(QALCULATE_CFLAGS)
 * AC_SUBST(QALCULATE_LIBS) \endcode
@@ -133,17 +134,32 @@ typedef enum {
 	MESSAGE_ERROR
 } MessageType;
 
+///Message stages
+#define MESSAGE_STAGE_CONVERSION		-4
+#define MESSAGE_STAGE_CONVERSION_PARSING	-3
+#define MESSAGE_STAGE_CALCULATION		-2
+#define MESSAGE_STAGE_PARSING			-1
+#define MESSAGE_STAGE_UNSET			0
+
+///Message categories
+#define MESSAGE_CATEGORY_NONE		0
+#define MESSAGE_CATEGORY_PARSING	1
+
+
 /// A message with information to the user. Primarily used for errors and warnings.
 class CalculatorMessage {
   protected:
 	string smessage;
 	MessageType mtype;
+	int i_stage, i_cat;
   public:
-	CalculatorMessage(string message_, MessageType type_ = MESSAGE_WARNING);
+	CalculatorMessage(string message_, MessageType type_ = MESSAGE_WARNING, int cat_ = MESSAGE_CATEGORY_NONE, int stage_ = MESSAGE_STAGE_UNSET);
 	CalculatorMessage(const CalculatorMessage &e);
 	string message() const;
-	const char* c_message() const;	
+	const char* c_message() const;
 	MessageType type() const;
+	int stage() const;
+	int category() const;
 };
 
 #include <libqalculate/MathStructure.h>
@@ -172,6 +188,8 @@ struct Element {
 
 #define UFV_LENGTHS	20
 
+class Calculate_p;
+
 /// The almighty calculator class.
 /** The calculator class is responsible for loading functions, variables and units, and keeping track of them, as well as parsing expressions and much more. A calculator object must be created before any other Qalculate! class is used. There should never be more than one calculator object, accessed with CALCULATOR. 
 *
@@ -198,10 +216,7 @@ class Calculator {
 	
 	vector<DataSet*> data_sets;
 	
-	unordered_map<size_t, MathStructure*> id_structs;
-	unordered_map<size_t, bool> ids_p;
-	vector<size_t> freed_ids;	
-	size_t ids_i;
+	class Calculator_p *priv;
 	
 	vector<string> signs;	
 	vector<string> real_signs;
@@ -218,6 +233,7 @@ class Calculator {
 	string NAME_NUMBER_PRE_S, NAME_NUMBER_PRE_STR, DOT_STR, DOT_S, COMMA_S, COMMA_STR, ILLEGAL_IN_NAMES, ILLEGAL_IN_UNITNAMES, ILLEGAL_IN_NAMES_MINUS_SPACE_STR;
 
 	bool b_argument_errors;
+	int current_stage;
 
 	time_t exchange_rates_time, exchange_rates_check_time;
 	bool b_exchange_rates_used, b_exchange_rates_warning_enabled;
@@ -278,6 +294,7 @@ class Calculator {
 	MathFunction *f_diff, *f_integrate, *f_solve, *f_multisolve;
 	MathFunction *f_error, *f_warning, *f_message, *f_save, *f_load, *f_export, *f_title;
 	MathFunction *f_register, *f_stack;
+	MathFunction *f_plot;
 	MathFunction *f_uncertainty;
 	
 	Unit *u_rad, *u_gra, *u_deg, *u_euro, *u_kelvin, *u_celsius, *u_fahrenheit;
@@ -338,7 +355,7 @@ class Calculator {
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	* @returns The result of the calculation.
 	*/
-	MathStructure calculate(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);	
+	MathStructure calculate(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	int testCondition(string expression);
 	//@}
 
@@ -588,7 +605,7 @@ class Calculator {
 	* @param always_convert ...
 	* @returns Converted value.
 	*/
-	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true);
+	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
 	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);
 	MathStructure convertTimeOut(string str, Unit *from_unit, Unit *to_unit, int msecs, const EvaluationOptions &eo = default_evaluation_options);
 	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);	
@@ -596,6 +613,7 @@ class Calculator {
 	Unit *getBestUnit(Unit *u, bool allow_only_div = false);
 	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
 	MathStructure convertToCompositeUnit(const MathStructure &mstruct, CompositeUnit *cu, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true);
+	MathStructure convertToMixedUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
 	//@}
 
 	/** Used by the UI to find unit category for a mathematical expression.*/
@@ -842,10 +860,13 @@ class Calculator {
 
 	/** @name Functions for message handling. */
 	//@{
+	void error(bool critical, int message_category, const char *TEMPLATE,...);
 	void error(bool critical, const char *TEMPLATE,...);
 	/** Put a message in the message queue. 
 	*/
+	void message(MessageType mtype, int message_category, const char *TEMPLATE,...);
 	void message(MessageType mtype, const char *TEMPLATE,...);
+	void message(MessageType mtype, int message_category, const char *TEMPLATE, va_list ap);
 	/** Returns the first message in queue.
 	*/
 	CalculatorMessage *message();
